import { IFormSetting } from '.';
import { ICrossFieldConditions, IFormLayout, IFormV2 } from './models';
import { EnumType, ICrossFieldValidations } from './modelsCommon';
import { IFormSettings, IFormParsed } from './modelsParsed';

export const parseFormV2 = <T extends EnumType>(form: IFormV2<T>): IFormParsed<T> => {
  const {
    submitButtonText,
    cancelButtonText,
    validationObject,
    crossFieldValidations,
    crossFieldConditions,
    formLayout,
    children,
  } = form;

  const allFieldNames = Object.keys(validationObject.properties) as T[];

  const formOptions = allFieldNames.reduce((acc, fieldName) => {
    const fieldLayout = formLayout[fieldName] || {
      md: 12,
      sm: 12,
      xs: 12,
    };

    const formSetting: IFormSetting = { fieldLayout };

    if (crossFieldValidations?.[fieldName]) {
      formSetting.crossFieldValidations = crossFieldValidations[fieldName];
    }

    if (crossFieldConditions?.[fieldName]) {
      formSetting.crossFieldConditions = crossFieldConditions?.[fieldName];
    }

    acc[fieldName] = formSetting;

    return acc;
  }, {} as IFormSettings<T>);

  const parsedForm: IFormParsed<T> ={
    formatVersion: 'parsed',
    jsonSchema: validationObject,
    formSettings: formOptions,
    children,
    submitButtonText,
  };
  
  if(cancelButtonText){
    parsedForm.cancelButtonText=cancelButtonText
  }

  return parsedForm
};

export const parseToFormV2 = <T extends EnumType>(form: IFormParsed<T>): IFormV2<T> => {
  const { submitButtonText, cancelButtonText, formSettings, jsonSchema, children } = form;

  const allFieldNames = Object.keys(formSettings) as T[];

  const crossFieldValidations: ICrossFieldValidations<T> = {};

  const crossFieldConditions: ICrossFieldConditions<T> = {};

  const formLayout: IFormLayout<T> = {};

  allFieldNames.forEach((key) => {
    const keyCrossFieldConditions=formSettings[key]?.crossFieldConditions
    const keyCrossFieldValidations=formSettings[key]?.crossFieldValidations
    if(keyCrossFieldConditions){
      crossFieldConditions[key] = keyCrossFieldConditions;
    }

    if(keyCrossFieldValidations){
      crossFieldValidations[key] = keyCrossFieldValidations;
    }
    
    formLayout[key] = formSettings[key]?.fieldLayout || {
      md: 12,
      sm: 12,
      xs: 12,
    };
  });

  const formV2: IFormV2<T>={
    formatVersion: 2,
    crossFieldConditions,
    crossFieldValidations,
    formLayout,
    validationObject: jsonSchema,
    children,
    submitButtonText,
  };

  if(cancelButtonText){
    formV2.cancelButtonText=cancelButtonText
  }

  return formV2
};
